------------------------
-- Orientation
------------------------
-- import
local class                 = require "hp/lang/class"
local table                 = require "hp/lang/table"
local MOAIPropUtil          = require "hp/util/MOAIPropUtil"
local PropertyUtil          = require "hp/util/PropertyUtil"
require "global_variables"

-- class
local M                     = class(MOAIPropUtil)
local MOAIPropInterface     = MOAIProp.getInterfaceTable()

-- constraints
M.MOAI_CLASS                = MOAIProp

--------------------------------------------------------------------------------
-- Instance generating functions.<br>
-- Unlike an ordinary class, and based on the MOAI_CLASS.<br>
-- To inherit this function is not recommended.<br>
-- @param ... params.
-- @return instance.
--------------------------------------------------------------------------------
function M:new(...)
    local obj = self.MOAI_CLASS.new()
    table.copy(self, obj)

    if obj.init then
        obj:init(...)
    end

    obj.new = nil
    obj.init = nil
    
    obj.offsetX, obj.offsetY = 0, 0
    
    return obj
end

--------------------------------------------------------------------------------
-- Set the name.
-- @param name Object name.<br>
--------------------------------------------------------------------------------
function M:setView(view)
  self.view = view
end  

function M:getView()
  return self.view
end  

function M:getAdjustedViewWidth()
  local viewWidth = self.view:getWidth()
  if CurrentOrientation ==  Orientation.LANDSCAPE_LEFT or CurrentOrientation ==  Orientation.LANDSCAPE_RIGHT then
     viewWidth = self.view:getHeight()
  end 
  return viewWidth
end  

function M:getAdjustedViewHeight()
  local viewHeight = self.view:getHeight()
  if CurrentOrientation ==  Orientation.LANDSCAPE_LEFT or CurrentOrientation ==  Orientation.LANDSCAPE_RIGHT then
     viewHeight = self.view:getWidth()
  end 
  return viewHeight
end  

function M:getAdjustedViewSize()
  return self:getAdjustedViewWidth(), self:getAdjustedViewHeight()
end

function M:getViewSize()
  return self.view:getWidth(), self.view:getHeight()
end

function M:setOffset(x, y)
  self.offsetX, self.offsetY = x, y
end  

function M:getAdjustedPos(x, y)
  return x + self.offsetX, y + self.offsetY
end

function M:getAdjustedTapPoint(x, y)
  local tmpX, tmpY = x, y
  if CurrentOrientation == Orientation.PORTRAIT_UP then
    tmpX, tmpY =  x, y
  elseif CurrentOrientation == Orientation.PORTRAIT_DOWN then
    tmpX, tmpY =   self:getAdjustedViewWidth() - x, self:getAdjustedViewHeight() - y
  elseif CurrentOrientation == Orientation.LANDSCAPE_LEFT then
    tmpX, tmpY =  y, self:getAdjustedViewHeight() - x
  elseif CurrentOrientation == Orientation.LANDSCAPE_RIGHT then
    tmpX, tmpY = self:getAdjustedViewWidth() - y, x
  end
  return tmpX, tmpY
end  

-- input corner: phical corner of the phone
-- output: corner in current orientation
function M:getAdjustedCorner(corner) 
  if CurrentOrientation == Orientation.PORTRAIT_UP then
    return corner
  elseif CurrentOrientation == Orientation.PORTRAIT_DOWN then
    if corner == SCREEN_CORNER.TOP_LEFT then
      return SCREEN_CORNER.BOT_RIGHT
    elseif corner == SCREEN_CORNER.TOP_RIGHT then
      return SCREEN_CORNER.BOT_LEFT
    elseif corner == SCREEN_CORNER.BOT_LEFT then
      return SCREEN_CORNER.TOP_RIGHT
    elseif corner == SCREEN_CORNER.BOT_RIGHT then
      return SCREEN_CORNER.TOP_LEFT
    else
      return corner  
    end      
  elseif CurrentOrientation == Orientation.LANDSCAPE_LEFT then
    if corner == SCREEN_CORNER.TOP_LEFT then
      return SCREEN_CORNER.BOT_LEFT
    elseif corner == SCREEN_CORNER.TOP_RIGHT then
      return SCREEN_CORNER.TOP_LEFT
    elseif corner == SCREEN_CORNER.BOT_LEFT then
      return SCREEN_CORNER.BOT_RIGHT
    elseif corner == SCREEN_CORNER.BOT_RIGHT then
      return SCREEN_CORNER.TOP_RIGHT
    else
      return corner  
    end
  elseif CurrentOrientation == Orientation.LANDSCAPE_RIGHT then
    if corner == SCREEN_CORNER.TOP_LEFT then
      return SCREEN_CORNER.TOP_RIGHT
    elseif corner == SCREEN_CORNER.TOP_RIGHT then
      return SCREEN_CORNER.BOT_RIGHT
    elseif corner == SCREEN_CORNER.BOT_LEFT then
      return SCREEN_CORNER.TOP_LEFT
    elseif corner == SCREEN_CORNER.BOT_RIGHT then
      return SCREEN_CORNER.BOT_LEFT
    else
      return corner  
    end
  else 
    return corner  
  end
end  

-- input corner: corner in current orientation
-- output: phical corner of the phone
function M:getPhysicalCorner(corner)
  if self:getAdjustedCorner(SCREEN_CORNER.TOP_LEFT) == corner then
    return SCREEN_CORNER.TOP_LEFT
  elseif self:getAdjustedCorner(SCREEN_CORNER.TOP_RIGHT) == corner then
    return SCREEN_CORNER.TOP_RIGHT
  elseif self:getAdjustedCorner(SCREEN_CORNER.BOT_LEFT) == corner then
    return SCREEN_CORNER.BOT_LEFT
  else 
    return SCREEN_CORNER.BOT_RIGHT
  end  
end

function M:getRotatedMoveXY(moveX, moveY)
  if CurrentOrientation == Orientation.PORTRAIT_UP then
    return moveX, moveY
  elseif CurrentOrientation == Orientation.PORTRAIT_DOWN then
    return -moveX, -moveY
  elseif CurrentOrientation == Orientation.LANDSCAPE_LEFT then
    return moveY, -moveX
  elseif CurrentOrientation == Orientation.LANDSCAPE_RIGHT then
    return -moveY, moveX
  else --default
    return moveX, moveY
  end   
end 



return M