
------------------------
-- DEBUG_FLAG
------------------------

DEBUG_USE_DEFAULT_IMAGE = false
DEBUG_DISABLE_VIDEO = false
DEBUG_DISABLE_ORIENTATION = false
DEBUG_FORCE_AS_INITIATOR = false
DEBUG_TOGGLE_SHARE_MORE = false
DEBUG_FORCE_GARBAGE_COLLECTION = false
DEBUG_SHOW_VERSION_NUMBER = false
DEBUG_TRUN_OFF_BACK_BUTTON_INTERCEPTION = false
DEBUG_NO_SCALE_ON_RESOLUTION = false
DEBUG_SHOW_MEMORY_USAGE = true
------------------------
-- CONSTANTS
------------------------
DEBUG_VERSION_NUMBER = "v53"
MAX_PHOTO_COUNT = 50 -- uses can share at most max MAX_PHOTO_COUNT photos in a session
RELOAD_PHOTO_SPAN = 3
CURRENT_PROTOCOL_VERSION = 1
PHOTO_TRANSFER_THRESHOLD = 6 -- if the AvgTransferTime is bigger than this value, stop video sending/receiving
MEGA = 1024 * 1024
KILO = 1024
THUMBNAIL_SIZE = 128
-- fields in photo meta data
COUNT = "COUNT"
ID = "ID"
URL = "URL"
STATUS = "STATUS"
DIRECTION = "DIRECTION"
PATH = "PATH"
PAGE = "PAGE"
IMAGE_SPRITE = "IMAGE_SPRITE"
LOADING_VIEW = "LOADING_VIEW"
IMAGE_THUMBNAIL = "IMAGE_THUMBNAIL"
IMAGE_CONTAINER = "IMAGE_CONTAINER"
TRANSFER_START_TIME = "TRANSFER_START_TIME"
PHOTO_LOADED = "PHOTO_LOADED"

PX = "PX" -- screen based
PY = "PY" -- screen based

ID_FOR_SHARE_MORE_PAGE = "ID_FOR_SHARE_MORE_PAGE"
ID_FOR_INTOR_VIEW_PAGE = "ID_FOR_INTOR_VIEW_PAGE"
INTRO_VIEW_PAGE = 0 -- photo pages start at 1
-- photo status
STATUS_IDLE = "STATUS_IDLE"
STATUS_SENDING = "STATUS_SENDING"
STATUS_SENT = "STATUS_SENT"
STATUS_FAIL_TO_SEND = "STATUS_FAIL_TO_SEND"
STATUS_RECEIVING = "STATUS_RECEIVING"
STATUS_RECEIVED = "STATUS_RECEIVED"
STATUS_FAIL_TO_RECEIVE = "STATUS_FAIL_TO_RECEIVE"

-- photo direction
DIRECTION_SENDER = "DIRECTION_SENDER"
DIRECTION_RECEIVER = "DIRECTION_RECEIVER"

-- message types
PICK_PHOTO_MESSAGE = "PICK_PHOTO_MESSAGE"
CANCEL_PICKING_PHOTO_MESSAGE = "CANCEL_PICKING_PHOTO_MESSAGE"
ADD_PHOTO_MESSAGE = "ADD_PHOTO_MESSAGE"
ADD_PHOTO_LIST_MESSAGE = "ADD_PHOTO_LIST_MESSAGE"
UPDATE_PHOTO_MESSAGE = "UPDATE_PHOTO_MESSAGE"
SCROLL_PHOTO_MESSAGE = "SCROLL_PHOTO_MESSAGE"
ACK_MESSAGE = "ACK_MESSAGE" -- send by the receiver to inform the sender that the photo is received
POINTER_MESSAGE = "POINTER_MESSAGE"
STOP_SEND_VIDEO_MESSAGE = "STOP_SEND_VIDEO_MESSAGE"
RESUME_SEND_VIDEO_MESSAGE = "RESUME_SEND_VIDEO_MESSAGE"
REQUEST_PHOTO_LIST_MESSAGE = "REQUEST_PHOTO_LIST_MESSAGE"
-- message payload
MESSAGE_TYPE = "MESSAGE_TYPE"
PROTOCOL_VERSION = "PROTOCOL_VERSION"
PHOTO_ID_LIST = "PHOTO_ID_LIST"
SCROLL_ID = "SCROLL_ID"
-- os type
OS_ANDROID = "OS_ANDROID"
OS_IOS = "OS_IOS"
OS_OTHER = "OS_OTHER"

-- screen corner type
SCREEN_CORNER = {
  TOP_LEFT = "top_left",
  TOP_RIGHT = "top_right",
  BOT_LEFT = "bottom_left",
  BOT_RIGHT = "botttom_right"
}

-- device orientation
Orientation = { 
	PORTRAIT_UP = "PORTRAIT_UP", 
	PORTRAIT_DOWN = "PORTRAIT_DOWN",
	LANDSCAPE_LEFT = "LANDSCAPE_LEFT",
	LANDSCAPE_RIGHT = "LANDSCAPE_RIGHT",
}

-- image scaling
MAX_IMAGE_LENGTH = 640
JPEG_COMPRESS_RATIO = 0.3

-- do not send/receive video while sending/receiving pictures on following devices
-- https://github.com/erica/uidevice-extension/blob/master/UIDevice-Hardware.m
LOW_SPEED_DEVICES = {
  "IOS_iPhone1",
  "IOS_iPhone2",
  "IOS_iPhone3",
  "IOS_iPod1",
  "IOS_iPod2",
  "IOS_iPod3",
  "IOS_iPod4",
  "IOS_iPad1",
}

function setupOSType()
  if MOAIImagePickerIOS ~= nil then
    OSType = OS_IOS
  elseif MOAIMultiImagePickerAndroid ~= nil then
    OSType = OS_ANDROID
  else
    OSType = OS_OTHER
  end
end

setupOSType()

-- UI element
function setupTargetScreenSize()
  local ratio = Application.screenWidth/Application.screenHeight
  local max = math.max(Application.screenWidth, Application.screenHeight)
  if DEBUG_NO_SCALE_ON_RESOLUTION then
    TARGET_SCREEN_WIDTH = Application.screenWidth
    TARGET_SCREEN_HEIGHT = Application.screenHeight
  elseif ratio == 2 / 3 then
    TARGET_SCREEN_WIDTH = 640
    TARGET_SCREEN_HEIGHT = 960
  elseif ratio == 3 / 4 then
    TARGET_SCREEN_WIDTH = 768
    TARGET_SCREEN_HEIGHT = 1024
  elseif ratio == 640 / 1136 then
    TARGET_SCREEN_WIDTH = 640
    TARGET_SCREEN_HEIGHT = 1136
  elseif max > 960 and max < 1300 then
    TARGET_SCREEN_WIDTH = Application.screenWidth
    TARGET_SCREEN_HEIGHT = Application.screenHeight
  elseif max >= 1300 then -- for xhdpi devices
    -- FIX: TONE-3356
    -- [Photo Sharing] On Android with higher resolution screens - text and buttons are very small
    TARGET_SCREEN_WIDTH = 640
    TARGET_SCREEN_HEIGHT = Application.screenHeight/Application.screenWidth * 640
  else
    -- default value
    TARGET_SCREEN_WIDTH = 640
    TARGET_SCREEN_HEIGHT = 960
  end    
  print("setupTargetScreenSize() screen width: ".. Application.screenWidth .. ", screen height: " ..Application.screenHeight .. ", target width: " .. TARGET_SCREEN_WIDTH .. ", target height: " .. TARGET_SCREEN_HEIGHT)
end

CLOSE_BUTTON_STYLES = {
    normal = {
        skin = "../assets/close.png",
        skinColor = {1, 1, 1, 1.0},
    },
    selected = {
        skin = "../assets/close.png",
        skinColor = {0.5, 0.5, 0.5, 0.8},
    },
    over = {
        skin = "../assets/close.png",
        skinColor = {0.5, 0.5, 0.5, 0.8},
    },
    disabled = {
        skinColor = "../assets/close.png",
    },    
}

ADD_MORE_ICON_NORMAL_STYLES = {
    normal = {
        skin = "../assets/add_btn_white.png",
        skinColor = {1, 1, 1, 1.0},
    },
    selected = {
        skin = "../assets/add_btn_white.png",
        skinColor = {0.5, 0.5, 0.5, 0.8},
    },   
}

ADD_MORE_ICON_HIGHLIGHTED_STYLES = {
    normal = {
        skin = "../assets/add_btn_orange.png",
        skinColor = {1, 1, 1, 1.0},
    },
    selected = {
        skin = "../assets/add_btn_orange.png",
        skinColor = {0.5, 0.5, 0.5, 0.8},
    },   
}

MUTE_BUTTON_STYLES = {
    normal = {
        skin = "../assets/unmute_icon.png",
        skinColor = {1, 1, 1, 1.0},
    },
    selected = {
        skin = "../assets/mute_icon.png",
        skinColor = {1, 1, 1, 1.0},
    },
    over = {
        skinColor = {0.5, 0.5, 0.5, 0.8},
    },
}

SHARE_MORE_BUTTON_STYLES = {
    normal = {
        skin = "../assets/btn_orange.png",
        skinClass = NinePatch,
        skinColor = {1, 1, 1, 1},
        textSize = 38,
        textColor = {1, 0.94, 0.93, 1},
        textPadding = {0, 0, 0, 0},
    },
    selected = {
        skin = "../assets/btn_orange.png",
        textSize = 40,
        textColor = {0.65, 0.14, 0.08, 1},
        textPadding = {0, 0, 0, 0},
    },
    over = {
        skin = "../assets/btn_orange.png",
    },
    disabled = {
        skin = "../assets/btn_orange.png",
        textColor = {0.5, 0.5, 0.5, 1},
    },
}

CANCEL_BUTTON_STYLES = {
    normal = {
        skin = "../assets/btn_grey.png",
        skinClass = NinePatch,
        skinColor = {1, 1, 1, 1},
        --font = "Roboto",
        textSize = 32,
        textColor = {254/255, 240/255, 237/255, 1},
        textPadding = {10, 5, 10, 8},
    },
    selected = {
        skin = "../assets/btn_grey.png",
        textSize = 32,
        textColor = {0.55, 0.55, 0.54, 1},
        textPadding = {10, 5, 10, 8},
    },
    over = {
        skin = "../assets/btn_grey.png",
    },
    disabled = {
        skin = "../assets/btn_grey.png",
        textColor = {0.5, 0.5, 0.5, 1},
    },
}

OK_BUTTON_STYLES = {
    normal = {
        skin = "../assets/btn_orange.png",
        skinClass = NinePatch,
        skinColor = {1, 1, 1, 1},
        --font = "Roboto",
        textSize = 32,
        textColor = {254/255, 240/255, 237/255, 1},
        textPadding = {10, 5, 10, 8},
    },
    selected = {
        skin = "../assets/btn_orange.png",
        textSize = 32,
        textColor = {0.65, 0.14, 0.08, 1},
        textPadding = {10, 5, 10, 8},
    },
    over = {
        skin = "../assets/btn_orange.png",
    },
    disabled = {
        skin = "../assets/btn_orange.png",
        textColor = {0.5, 0.5, 0.5, 1},
    },
}

local pageNumberBG = OSType == OS_ANDROID and "../assets/paging_bg_android.png" or "../assets/photo_counter_bg@2x.png"

PAGE_NUMBER_STYLES = {
    normal = {
        skin = pageNumberBG,
        textSize = 28,
        textColor = {1, 0.94, 0.93, 1},
        skinColor = {1, 1, 1, 1.0},
    },
    selected = {
        skin = pageNumberBG,
        textSize = 28,
        textColor = {1, 0.94, 0.93, 1},
        skinColor = {1, 1, 1, 1.0},
    }
}

IMAGE_CONTAINER = "IMAGE_CONTAINER"
IMAGE_SUB_CONTAINER = "IMAGE_SUB_CONTAINER"
IMAGE_SPRITE = "IMAGE_SPRITE"
LAST_PAGE = "LAST_PAGE"

-- others
MAX_SHARE_PHOTO_DOWNLOAD_TOKEN_COUNT = 2;
MAX_SHARE_PHOTO_UPLOAD_TOKEN_COUNT = 2;

LANGUAGE = string.lower(MOAIEnvironment.languageCode)
if string.len(LANGUAGE) == 5 then
  LANGUAGE = string.sub(LANGUAGE, 1, 2)
end  
---
BUTTON_WIDTH = 100
BUTTON_HEIGHT = 100

ICON_WIDTH = 114

------------------------
-- Global Variables
------------------------
MOAIImagePicker = MOAIImagePickerIOS or MOAIMultiImagePickerAndroid

MOAILocalization = MOAILocalizationIOS or MOAILocalizationAndroid

Call = nil
VideoControl = nil
AudioControl = nil
PIP_MARGIN = 10

PhotoTable = {}
CurrentPhotoCount = 0
CurrentPage = INTRO_VIEW_PAGE
CurrentScrollId = 0
PhotoCountFromMe = 0
PhotoCountToMe = 0

CurrentDownloadToken = MAX_SHARE_PHOTO_DOWNLOAD_TOKEN_COUNT
CurrentUploadToken = MAX_SHARE_PHOTO_UPLOAD_TOKEN_COUNT

PeerId = ""
CurrentPhotoId = 0

CurrentOrientation = Orientation.PORTRAIT_UP
function setCurrentOrientation(orientation)
  CurrentOrientation = orientation
end  

Myself = nil
PeerInfo = nil

FlagDisplayExitDialog = false
FlagShowedScrollerIndicator = false
FlagFirstEnterScreen = true

OSType = nil

PIP_CORNER = SCREEN_CORNER.BOT_LEFT

ImagePickerVisible = false

AvgTransferTime = 0

IsLowSpeedDevice = false
IsPeerPickingPhoto = false
PeerSelectingPhotoCount = 0
ShouldShowPopup = false
ShouldExitIfNoPhotoPicked = false

LogBeenShareMorePhoto = false
LogViewedAllPhotos = false
LogClickPickPhotoButton = false
LogExitBy = "peer"